from __future__ import with_statement
import sys

try:
    import inspect
    import numpy
    import os
    import re
    import reflex
    from pipeline_product import PipelineProduct
    import pipeline_display
    import reflex_plot_widgets
    import matplotlib.gridspec as gridspec
    from matplotlib.text import Text
    from matplotlib.pyplot import cm
    from pylab import *

    import_success = True
except ImportError:
    import_success = False
    print "Error importing modules pyfits, wx, matplotlib, numpy"

def paragraph(text, width=None):
    """ wrap text string into paragraph
       text:  text to format, removes leading space and newlines
       width: if not None, wraps text, not recommended for tooltips as
              they are wrapped by wxWidgets by default
    """
    import textwrap
    if width is None:
        return textwrap.dedent(text).replace('\n', ' ').strip()
    else:
        return textwrap.fill(textwrap.dedent(text), width=width)

class DataPlotterManager(object):
    # static members
    recipe_name = "kmos_combine"
    combined_cat = "COMBINE_SCI_RECONSTRUCTED"
    recons_cat = "SCI_RECONSTRUCTED"
    oh_spec_cat = "OH_SPEC"
    cores = ['r', 'g', '0.50', 'c', 'm', 'y', 'b']
    labels_view=['Display input and output spectra', 'Display output spectra', 'Display input spectra']
    vs_state = 0
    selected_file_idx = 0

    def setCurrentParameterHelper(self, helper):
        self.param_helper = helper

    def setWindowTitle(self):
        return self.recipe_name+"_interactive"

    def setInteractiveParameters(self):
        return [
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="name",
                    group="Comb.", description="Name of the object to combine. Empty[default] means all."),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="fmethod",
                    group="Comb.", description="Fitting Method (gauss, moffat)"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="flux",
                    group="Comb.", description="Apply flux conservation"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="edge_nan",
                    group="Comb.", description="Set borders of cubes to NaN before combining them"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="cmethod",
                    group="Comb.", description="Combination Method (average, median, sum, min_max, ksigma)"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="cpos_rej",
                    group="Comb.", description="The positive rejection threshold"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="cneg_rej",
                    group="Comb.", description="The negative rejection threshold"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="citer",
                    group="Comb.", description="The number of iterations"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="cmax",
                    group="Comb.", description="The number of maximum pixel values"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="cmin",
                    group="Comb.", description="The number of minimum pixel values"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="skipped_frames",
                    group="Comb.", description="Comma-separated list of R:I duplets/'labels' indexing the frames to skip. R:I means the Rth RAW frame from the Ith IFU will be ignored. Empty[default] skips none."),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="ifus",
                    group="Other", description="The indices of the IFUs to combine"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="method",
                    group="Other", description="The shifting method"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="filename",
                    group="Other", description="The path to the file with the shift vectors"),
        ]

    def readFitsData(self, fitsFiles):
        # Initialise
        self.comb_files = []    # List of Dict (1 per comb. file) - Keys : FILENAME, USEDIFUS, SKIPPEDIFUS, CRPIX3, CRVAL3, CDELT3, UNIT, Spectrum, Collapsed, in_ifus
                                # in_fus: List of Dict (1 per used IFUS) - Keys : REC_FILENAME, rec_im_nums, ifu_nb, CRPIX3, CRVAL3, CDELT3, UNIT, Spectrum
        self.oh_spec = dict()   # Dict - Keys : Found, CRVAL1, CDELT1, Spectrum 

        # Loop on all FITS files 
        self.oh_spec["Found"] = False
        for f in fitsFiles:
            # Use OH_SPEC if found
            if f.category == self.oh_spec_cat :
                oh_spec_file = PipelineProduct(f)
                self.oh_spec["Found"] = True
                self.oh_spec["CRVAL1"] = oh_spec_file.all_hdu[1].header["CRVAL1"]
                self.oh_spec["CDELT1"] = oh_spec_file.all_hdu[1].header["CDELT1"]
                self.oh_spec["Spectrum"] = oh_spec_file.all_hdu[1].data

            # For each COMBINED image
            if f.category == self.combined_cat :
                combined_file = PipelineProduct(f)
                filename = os.path.basename(f.name)

                # Create a Dictionary per file
                my_file_dict = dict()
                
                # STORE the combined file name
                my_file_dict["FILENAME"] = filename
                
                # STORE the Used reconstructed files IFUS : "1:3,2:5,..."
                combined_primary = combined_file.all_hdu[0]
                
                # Support case where keyword is missing
                if 'ESO PRO USEDIFUS' in combined_primary.header :
                    my_file_dict["USEDIFUS"] = combined_primary.header['ESO PRO USEDIFUS']
                else :
                    my_file_dict["USEDIFUS"] = ""
                if 'ESO PRO SKIPPEDIFUS' in combined_primary.header :
                    my_file_dict["SKIPPEDIFUS"] = combined_primary.header['ESO PRO SKIPPEDIFUS']
                else :
                    my_file_dict["SKIPPEDIFUS"] = ""
               
                # Store the COMBINE Spectrum and Image
                combined_ext = combined_file.all_hdu[1]            
                naxis = combined_ext.header['NAXIS']
                if (naxis == 3):
                    # Get Keyword infos  
                    my_file_dict["CRPIX3"] = combined_ext.header['CRPIX3']
                    my_file_dict["CRVAL3"] = combined_ext.header['CRVAL3']
                    my_file_dict["CDELT3"] = combined_ext.header['CDELT3']
                    my_file_dict["UNIT"]   = combined_ext.header['ESO QC CUBE_UNIT']
                    
                    # Fill Spectrum
                    my_file_dict["Spectrum"] = []
                    for cube_plane in combined_ext.data:
                        cube_plane_nan_free = cube_plane[~numpy.isnan(cube_plane)]
                        if (len(cube_plane_nan_free) > 0):
                            mean = cube_plane_nan_free.mean()
                        else:
                            mean = numpy.nan
                        my_file_dict["Spectrum"].append(mean)

                    # Fill Collapsed Image
                    collapsed_frame_ext = self._get_collapsed_ext(fitsFiles, f)
                    if collapsed_frame_ext:
                        my_file_dict["Collapsed"] = collapsed_frame_ext.data
 
                ### Parse R:I from CSV string ##################
                rec_im_nums = dict()
                ifu_nums = dict()
                skipped = dict()
                # Store used IFUs from the FITS keyword
                duplet_seq = my_file_dict["USEDIFUS"]
                duplet_seq.strip()
                
                if len(duplet_seq) > 1 :
                    duplet_seq = duplet_seq.split(",")
                    for kindx in range(0, len(duplet_seq)):
                        xpto = duplet_seq[kindx]
                        aux = xpto.split(":")
                        rec_im_nums[kindx] = int( aux[0] )
                        ifu_nums[kindx] = int( aux[1] )
                        skipped[kindx] = 0
                    
                # Store Skipped IFUs
                next_idx = len(duplet_seq)
                duplet_seq = my_file_dict["SKIPPEDIFUS"]
                duplet_seq.strip()
                if len(duplet_seq) > 1 :
                    duplet_seq = duplet_seq.split(",")
                    for skipped_idx in range(0, len(duplet_seq)):
                        xpto = duplet_seq[skipped_idx]
                        aux = xpto.split(":")
                        rec_im_nums[next_idx+skipped_idx] = int( aux[0] )
                        ifu_nums[next_idx+skipped_idx] = int( aux[1] )
                        skipped[next_idx+skipped_idx] = 1
                ### end ########################################
                
                # Get the Input IFUs for this combined image
                my_file_dict["in_ifus"] = []

                # Loop on all input fitsfiles
                countr = 0  
                for frec in fitsFiles:
                    if frec.category == self.recons_cat :
                        reconstructed_file = PipelineProduct(frec)
                        reconstructed_filename = os.path.basename(frec.name)
                        
                        # countr is the id of the current REC file amongst the REC files
                        countr += 1
                        
                        # Loop on the used IFUS to get the ones from the current REC file
                        for duplet_num in range(0, len(rec_im_nums) ) :   
                            r = rec_im_nums[duplet_num]

                            # Check if the current REC file is specified here
                            if r == countr :
                                # Create a dictionary per used IFU
                                my_used_ifus_dict = dict()

                                # STORE RECONSTRUCTED filename
                                my_used_ifus_dict["REC_FILENAME"] = reconstructed_filename
                                
                                i = ifu_nums[duplet_num]

                                # STORE the ifu nb / rec file idx / skipped info
                                my_used_ifus_dict["ifu_nb"] = i
                                my_used_ifus_dict["rec_im_nums"] = r
                                my_used_ifus_dict["skipped"] = skipped[duplet_num]
                                
                                # Store the spectrum in a dictionary
# TODO : ith IFU in ith extension ??
                                rec_ext = reconstructed_file.all_hdu[i]  
                                rec_naxis = rec_ext.header['NAXIS']
                                if (rec_naxis == 3):
                                    # Get Keyword infos  
                                    my_used_ifus_dict["CRPIX3"] = rec_ext.header['CRPIX3']
                                    my_used_ifus_dict["CRVAL3"] = rec_ext.header['CRVAL3']
                                    my_used_ifus_dict["CDELT3"] = rec_ext.header['CDELT3']
                                    my_used_ifus_dict["UNIT"]   = rec_ext.header['ESO QC CUBE_UNIT']
                                    
                                    # Fill Spectrum
                                    my_used_ifus_dict["Spectrum"] = []
                                    for cube_plane in rec_ext.data:
                                        cube_plane_nan_free = cube_plane[~numpy.isnan(cube_plane)]
                                        if (len(cube_plane_nan_free) > 0):
                                            mean = cube_plane_nan_free.mean()
                                        else:
                                            mean = numpy.nan
                                        my_used_ifus_dict["Spectrum"].append(mean)
                                        
                                # add this dictionary my_file_dict["in_ifus"]
                                my_file_dict["in_ifus"].append(my_used_ifus_dict)       

                # Append the dictionary to the files list
                self.comb_files.append(my_file_dict)
               
        # If proper files are there...
        if (len(self.comb_files) > 0):
            # Set the plotting functions
            self._add_subplots = self._add_subplots
            self._plot = self._data_plot
        else:
            self._add_subplots = self._add_nodata_subplots
            self._plot = self._nodata_plot

    # Returns the corresponding extension in the collapsed corresponding file 
    # The corresponding file must have the same file name as ref_file, prefixed with make_image_
    def _get_collapsed_ext(self, fitsFiles, comb_file):
        comb_file_name = os.path.basename(comb_file.name)
        # Loop on all FITS files 
        for f in fitsFiles:
            make_image_filename = os.path.basename(f.name)
            if (make_image_filename == "make_image_"+comb_file_name):
                collapsed_frame = PipelineProduct(f)
                return collapsed_frame.all_hdu[1]

    def addSubplots(self, figure):
        self._add_subplots(figure)

    #def addSubplots_SS(self, figure):
        #self._add_subplots_SS(figure)

    def plotProductsGraphics(self):
#        print self.param_helper("skipped_frames")
        self._plot()

    def plotWidgets(self) :
        #self.radiobutton.clear()
        
        widgets = list()

        labels=[]
        for idx in range(0, len(self.comb_files)):
            labels.append(self.comb_files[idx]["FILENAME"])
        #labels_view=['Combined', 'Classic', 'Spectra Stack']
        
        # Files Selector radiobutton
        self.radiobutton = reflex_plot_widgets.InteractiveRadioButtons(self.comb_files_selector, self.setFSCallback, labels, self.selected_file_idx, title='Files Selection (double-click Left Mouse Button)')
        widgets.append(self.radiobutton)
        
        self.plotVS_Widget()
        
        return widgets
        
    def plotVS_Widget(self) :
        self.view_select.clear()
        widgets = list()
        # View Selector radiobutton
        self.radiobutton_VS = reflex_plot_widgets.InteractiveRadioButtons(self.view_select, self.setVSCallback, self.labels_view, self.vs_state)
        self.view_select.text(0.5, 0.0, 'View Selection (Left Mouse Button)', verticalalignment='bottom', horizontalalignment='center', 
                              fontsize=12, fontweight='semibold')
        widgets.append(self.radiobutton_VS)
        return widgets

#def setCurrentParameterHelper(self, helper) :
#print helper('skipped_frames')

    def setFSCallback(self, filename) :
        for idx in range(0, len(self.comb_files)):
            if (self.comb_files[idx]["FILENAME"] == filename):
                self.selected_file_idx = idx        
        # Redraw spectrum
        self._plot_spectrum()
        # Redisplay image
        self._disp_image()
        # Redraw spectra stack
        self._disp_spectra_stack()
        
    def setVSCallback(self, chosen_view) :
        #print(chosen_view)

        if(chosen_view == self.labels_view[0]):    #if(chosen_view == 'Combined'):
            self.vs_state = 0
            self.addSubplots(self.my_figure)
            
            # Redraw spectrum
            self._plot_spectrum()
            # Redisplay image
            self._disp_image()
            # Redraw spectra stack
            self._disp_spectra_stack()
            
            # Redraw View Selection radiobutton panel
            self.plotWidgets()
            
        if(chosen_view == self.labels_view[1]):    #if(chosen_view == 'Classic'):
            self.vs_state = 1
            self._add_subplots_Clssc(self.my_figure)
            
            # Redraw spectrum
            self._plot_spectrum()
            # Redisplay image
            self._disp_image()
            
            # Redraw View Selection radiobutton panel
            self.plotWidgets()              
                    
        if(chosen_view == self.labels_view[2]):    #if(chosen_view == 'Spectra Stack'):
            self.vs_state = 2
            self._add_subplots_SS(self.my_figure)

            # Redraw spectra stack
            self._disp_spectra_stack()
            
            # Redraw View Selection radiobutton panel
            self.plotVS_Widget()  
    
        self.my_figure.canvas.draw()
            
    def _add_subplots_Clssc(self, figure):
        self.my_figure = figure
        self.my_figure.clear()
        figure.clear()
        
        gs = gridspec.GridSpec(4, 3)
        self.comb_files_selector = figure.add_subplot(gs[0:2, :])
        self.spec_plot = figure.add_subplot(     gs[2,   0:2])
        self.collapsed_img = figure.add_subplot( gs[2,   2])
        self.view_select = figure.add_subplot(   gs[3,   :])

    def _add_subplots_SS(self, figure):
        self.my_figure = figure
        self.my_figure.clear()
        figure.clear()
        
        gs = gridspec.GridSpec(4, 3)
        self.spec_stack = figure.add_subplot(    gs[0:3, :])
        self.view_select = figure.add_subplot(   gs[3,   :])
        
    def _add_subplots(self, figure):
        self.my_figure = figure
        self.my_figure.clear()
        figure.clear()
        #stop_here()
        gs = gridspec.GridSpec(4, 3)
        self.comb_files_selector = figure.add_subplot(gs[0:2, 0:2])
        self.spec_plot = figure.add_subplot(     gs[2,   0])
        self.collapsed_img = figure.add_subplot( gs[2,   1])
        self.spec_stack = figure.add_subplot(    gs[0:3, 2])
        self.view_select = figure.add_subplot(   gs[3,   :])

    def _data_plot_get_tooltip(self):
        return self.comb_files[self.selected_file_idx]["FILENAME"]

    def _disp_image(self):
        extension_dict = self.comb_files[self.selected_file_idx]
        self.collapsed_img.clear()
        imgdisp = pipeline_display.ImageDisplay()
        imgdisp.setAspect('equal')

        # Only if data
        if 'Collapsed' in extension_dict.keys():
            imgdisp.display(self.collapsed_img, "Collapsed image", self._data_plot_get_tooltip(), extension_dict["Collapsed"])
            self.collapsed_img.set_xlabel("pixels")   # item 6f
            self.collapsed_img.set_ylabel("pixels")   # item 6f 

    def _plot_spectrum(self):
        extension_dict = self.comb_files[self.selected_file_idx] 
        
        # Plot Spectrum (to be contd.)
        self.spec_plot.clear()
        specdisp = pipeline_display.SpectrumDisplay()
        specdisp.setLabels(r"$\lambda$["+"$\mu$m]" + " (blue: Observed; red: OH[arb. units])", 
                            self._process_label(extension_dict["UNIT"]) )
        
        if (self.oh_spec["Found"]):  # Placed before spectrum plotting, so it stays in the background
            # Overplot the OH spectrum
            pix = numpy.arange(len(self.oh_spec["Spectrum"]))
            wave = self.oh_spec["CRVAL1"] + pix * self.oh_spec["CDELT1"]
            oh_flux = self.oh_spec["Spectrum"] / numpy.nanmax(self.oh_spec["Spectrum"]) * 200*numpy.nanmax(extension_dict["Spectrum"]) 
            specdisp.overplot(self.spec_plot, wave, oh_flux, '#ED5D5D')
            #self.spec_plot.legend(('Observed', 'OH'))
            self.spec_plot.fill_between(wave, 0, oh_flux, color='#ED5D5D')
        
        # Define wave
        pix = numpy.arange(len(extension_dict["Spectrum"]))
        wave = extension_dict["CRVAL3"] + pix * extension_dict["CDELT3"]

        # Plot Spectrum (contd.)
        specdisp.display(self.spec_plot, "Spectrum", self._data_plot_get_tooltip(), wave, extension_dict["Spectrum"])

    def _disp_spectra_stack(self):        
        comb_dict = self.comb_files[self.selected_file_idx]
       
        spec_list = []
        ifus_list = []
        rec_idx_list = []
        skipped = []
        for ind in range( 0, len(comb_dict["in_ifus"]) ) :
            spec_list.append( comb_dict["in_ifus"][ind]["Spectrum"] )  
            ifus_list.append( comb_dict["in_ifus"][ind]["ifu_nb"] )
            rec_idx_list.append( comb_dict["in_ifus"][ind]["rec_im_nums"] )
            skipped.append( comb_dict["in_ifus"][ind]["skipped"] )

        # Define wave
        pix = numpy.arange(len(comb_dict["Spectrum"]))
        wave = comb_dict["CRVAL3"] + pix * comb_dict["CDELT3"]
        lower_xbound = np.nanmin(wave)
        upper_xbound = np.nanmax(wave)
        #debug
        #print("\n" + "np.nanmin(wave) = " + str(np.nanmin(wave)) + "\n")
        #print("\n" + "np.nanmax(wave) = " + str(np.nanmax(wave)) + "\n")
        
        # Plot Spectrum
        self.spec_stack.clear()
        sstackdisp = pipeline_display.SpectrumDisplay()
        sstackdisp.setLabels(r"$\lambda$ ["+"$\mu$m] ",  self._process_label(comb_dict["UNIT"]) )
        
        v_offset = 0.
        lower_ybound = 0.
        upper_ybound = 0.
        for indx in range(0, len(spec_list) ):
            c = self.cores[mod(indx, len(self.cores))]
            spectrum = spec_list[indx]
            spec_ = [x for x in spectrum if (np.isnan(x) == False)]
            #os_factor = 1.0e-16; v_offset = os_factor * (indx+0)   #<-fixed step
            med = np.median(spec_) if not np.isnan(np.median(spec_)) else 0
            mad = np.median(np.abs(spec_ - med)) if not np.isnan(np.median(spec_)) else 0.1  #Once NaN always NaN assumption forced because of isnan()
            os_factor = med
            v_offset += mad*10  # cumulative version
            if lower_ybound == 0 :
                lower_ybound = v_offset
            
            # Skipped Spectra in Black
            if (skipped[indx]) :
                c = 'black' ;

            spectrum_str = str(rec_idx_list[indx])+':'+str(ifus_list[indx])
            sstackdisp.overplot(self.spec_stack, wave, np.add(spectrum, v_offset), c )
            self.spec_stack.text(upper_xbound*1.01211366512 if self.vs_state==2 else upper_xbound*1.04411725927, 
                                 v_offset +os_factor, spectrum_str, verticalalignment='top', horizontalalignment='right', color=c, fontsize=10)
            upper_ybound = v_offset + mad*10
            
        if (lower_ybound != 0 and upper_ybound != 0) :
            self.spec_stack.set_ybound(lower=lower_ybound*1.15, upper=upper_ybound*1.02)
            self.spec_stack.set_xbound(lower=lower_xbound*1.00259742743, upper=upper_xbound)
        if self.vs_state == 2 :
            self.spec_stack.set_title("IFUs for '"+comb_dict["FILENAME"]+"' (skipped in black)", fontsize=12, fontweight='semibold') 
            self.spec_stack.set_xlabel(r"$\lambda$ ["+"$\mu$m]" + "   (duplet label syntax, <Recons. Image #> : <IFU #>)")
        else :
            self.spec_stack.set_title("IFUs (skipped in black)\n", fontsize=12, fontweight='semibold') 
            self.spec_stack.set_xlabel(r"$\lambda$ ["+"$\mu$m]")
        self.spec_stack.set_ylabel( self._process_label(comb_dict["UNIT"]) ) 

        if len(spec_list) == 0 :
            self.spec_stack.set_title("Missing USEDIFUS", fontsize=12, fontweight='semibold')

    def _process_label(self, in_label):
        # If known, 'pretty print' the label
        if (in_label == "ERG/sec/cm2/A"):
            return "erg sec" + r"$^{-1}$"+"cm" + r"$^{-2}$" + r"$\AA^{-1}$"
        else:
            return in_label
    
    def _data_plot(self):
        #~ # Initial file is the first one
        #~ self.selected_file_idx = 0
        
        # Draw Spectrum
        self._plot_spectrum()

        # Display image
        self._disp_image()
        
        # Display spectra stack
        self._disp_spectra_stack()

    def _add_nodata_subplots(self, figure):
        self.img_plot = figure.add_subplot(1,1,1)

    def _nodata_plot(self):
        # could be moved to reflex library?
        self.img_plot.set_axis_off()
        text_nodata = "Data not found. Input files should contain this" \
                       " type:\n%s" % self.combined_cat
        self.img_plot.text(0.1, 0.6, text_nodata, color='#11557c',
                      fontsize=18, ha='left', va='center', alpha=1.0)
        self.img_plot.tooltip = 'No data found'

#This is the 'main' function
if __name__ == '__main__':
    from reflex_interactive_app import PipelineInteractiveApp

    # Create interactive application
    interactive_app = PipelineInteractiveApp(enable_init_sop=True)

    # get inputs from the command line
    interactive_app.parse_args()

    #Check if import failed or not
    if not import_success:
        interactive_app.setEnableGUI(False)

    # Open the interactive window if enabled
    if interactive_app.isGUIEnabled():
        # Get the specific functions for this window
        dataPlotManager = DataPlotterManager()

        interactive_app.setPlotManager(dataPlotManager)
        interactive_app.showGUI()
    else:
        interactive_app.set_continue_mode()

    #Print outputs. This is parsed by the Reflex python actor to
    #get the results. Do not remove
    interactive_app.print_outputs()
    sys.exit()




import reflex 
import sys 
import os

try:
    import pyfits as fits
except:
    from astropy.io import fits as fits

import numpy as np
import time

def getw(header):
   N=header["NAXIS1"]
   CRPIX=header["CRPIX1"]
   CDELT=header["CDELT1"]
   CRVAL=header["CRVAL1"]
   starty=-(CRPIX-1.0)*CDELT+CRVAL
   w=np.arange(N)*CDELT + starty
   return w

if __name__ == '__main__':
  parser = reflex.ReflexIOParser()
  parser.add_input("-i", "--star_spc_in", dest="star_spc_in")
  parser.add_input("-j", "--response_in", dest="response_in")
  parser.add_output("-o", "--out_sof", dest="out_sof")

  inputs = parser.get_inputs()

  inputs_spc  = inputs.star_spc_in
  inputs_res  = inputs.response_in
  outputs = parser.get_outputs()

  files_res=inputs_res.files
  files_spc=inputs_spc.files

  #get the output directory
  pattern = '--products-dir'
  for arg in sys.argv:
     if arg.split("=")[0] == pattern:
         output_dir = arg.split("=")[1]

  new_name=list()
  output_category=list()


 
  #START PROCEDURE
  
  #read response if present
  IsResponsePresent = 0
  for file in files_res:
      hdu_res = fits.open(file.name)
      IsResponsePresent = 1
  
  
  #  if respone is present, do some stuff and create the output sof
  if IsResponsePresent == 1:
    c = 1
    for file in files_spc:
      
      hdu_spc = fits.open(file.name)
      newname = output_dir + '/STD_RESP_CORR_'+str(c)+'.fits'
      new_name.append(newname)
      
      for exten in range(1,49,2):
           data = hdu_spc[exten].data
           noise = hdu_spc[exten+1].data
           response_ =  hdu_res[exten].data
           error_response_ = hdu_res[exten+1].data
      
           if data != None:
              
              wdata = getw(hdu_spc[exten].header)
              wresponse = getw(hdu_res[exten].header)
              
              #I need to interpolate the response and error_response to the wavelength range and step of data and noise.
              response = np.interp(wdata,wresponse,response_)
              hdu_spc[exten].data = data / response
              
           if noise != None:
              wnoise = getw(hdu_res[exten+1].header) 
              #I need to interpolate the response and error_response to the wavelength range and step of data and noise.
              error_response =  np.interp(wnoise,wresponse,error_response_)
              hdu_spc[exten+1].data = ((noise/response)**2 + (data/response**2 * error_response)**2 )**0.5
              
      
      output_purpose=file.purposes       
      output_category.append(file.category)
      hdu_spc.writeto(newname,clobber=True)
      c = c+1

    #WRITE OUTPUT SOF
    files = list()
    output_datasetname=inputs_spc.datasetName
    for i in range(len(new_name)):
       files.append(reflex.FitsFile(new_name[i],output_category[i],None,output_purpose))
    newsof = reflex.SetOfFiles(output_datasetname,files)
    outputs.out_sof = newsof

  # If I did not have a response do nothing; output sof is equal to the input STD  sof 
  else:
    outputs.out_sof = inputs.star_spc_in

  # print outputs
  parser.write_outputs()

  sys.exit()



from __future__ import with_statement
import sys

try:
    import numpy
    import re
    import reflex
    from pipeline_product import PipelineProduct
    import pipeline_display
    import reflex_plot_widgets
    import matplotlib.gridspec as gridspec
    import_success = True
except ImportError:
    import_success = False
    print "Error importing modules pyfits, wx, matplotlib, numpy"

def paragraph(text, width=None):
    """ wrap text string into paragraph
       text:  text to format, removes leading space and newlines
       width: if not None, wraps text, not recommended for tooltips as
              they are wrapped by wxWidgets by default
    """
    import textwrap
    if width is None:
        return textwrap.dedent(text).replace('\n', ' ').strip()
    else:
        return textwrap.fill(textwrap.dedent(text), width=width)


class DataPlotterManager(object):
    # static members
    recipe_name = "kmos_std_star"
    star_spec_cat = "STAR_SPEC"
    telluric_cat = "TELLURIC"
    std_image_cat = "STD_IMAGE"
    y_scalefactor = 1000

    def setWindowTitle(self):
        return self.recipe_name+"_interactive"

    def setInteractiveParameters(self):
        return [
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="imethod",
                    group="Recons.", description="Interpolation Method (NN, lwNN, swNN, MS, CS)"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="xcal_interpolation",
                    group="Recons.", description="Interpolate xcal between rotator angles"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="b_samples",
                    group="Recons.", description="The number of samples in wavelength"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="b_start",
                    group="Recons.", description="The lowest wavelength"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="b_end",
                    group="Recons.", description="The highest wavelength"),

            reflex.RecipeParameter(recipe=self.recipe_name, displayName="mask_method",
                    group="Extr.", description="Extraction Method (optimal/integrated)"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="centre",
                    group="Extr.", description="Centre (integrated only)"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="radius",
                    group="Extr.", description="Radius (integrated only)"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="fmethod",
                    group="Extr.", description="Fitting Method (gauss, moffat)"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="neighborhoodRange",
                    group="Extr.", description="Range for Neighbors"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="flux",
                    group="Extr.", description="Apply flux conservation"),

            reflex.RecipeParameter(recipe=self.recipe_name, displayName="cmethod",
                    group="Comb.", description="Combination Method (average, median, sum, min_max, ksigma)"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="cpos_rej",
                    group="Comb.", description="The positive rejection threshold"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="cneg_rej",
                    group="Comb.", description="The negative rejection threshold"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="citer",
                    group="Comb.", description="The number of iterations"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="cmax",
                    group="Comb.", description="The number of maximum pixel values"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="cmin",
                    group="Comb.", description="The number of minimum pixel values"),
            
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="startype",
                    group="Star", description="The star spectral type (O, B, A, F, G)"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="magnitude",
                    group="Star", description="Star Magnitude"),
        ]

    def readFitsData(self, fitsFiles):
        self.frames = dict()
        for f in fitsFiles:
            self.frames[f.category] = PipelineProduct(f)

        # Two cases: the file category is found or not found.
        # Define the plotting functions in both cases
        if self.std_image_cat in self.frames and self.star_spec_cat in self.frames and self.telluric_cat in self.frames:
            # Get the wished files
            std_image = self.frames[self.std_image_cat]
            star_spec = self.frames[self.star_spec_cat]
            telluric = self.frames[self.telluric_cat]
           
            # Initialise
            self.star_data_extnames = []
            self.qc_std_trace = dict()
            self.qc_spat_res = dict()
            self.image_std = dict()
            self.image_std_avg = dict()
            self.image_std_stdev = dict()
            self.qc_thruput = dict()
            self.qc_zpoint = dict()
            self.crpix1 = dict()
            self.crval1 = dict()
            self.cdelt1 = dict()
            self.spec_data = dict()
            self.spec_noise = dict()
            
            # READ data
            self.qc_nr_std_stars = star_spec.all_hdu[0].header['ESO QC NR STD STARS']
            self.qc_thruput_mean = star_spec.all_hdu[0].header['ESO QC THRUPUT MEAN']
            self.qc_thruput_sdv  = star_spec.all_hdu[0].header['ESO QC THRUPUT SDV']
            
            # Loop on all extensions of std_image to find the std stars extensions
            for std_image_ext in std_image.all_hdu:
                naxis = std_image_ext.header['NAXIS']
                # NAXIS is 2 if there is an image, 0 otherwise
                if (naxis == 2):
                    # extname is like IFU.3.DATA 
                    extname = std_image_ext.header['EXTNAME']
                    self.star_data_extnames.append(extname)
                    self.qc_std_trace[extname] = std_image_ext.header['ESO QC STD TRACE']
                    self.qc_spat_res[extname] = std_image_ext.header['ESO QC SPAT RES']
                    self.image_std[extname] = std_image_ext.data
                    self.image_std_avg[extname] = numpy.average(std_image_ext.data)
                    self.image_std_stdev[extname] = numpy.std(std_image_ext.data)

                    # Get infos from star_spec using the extname
                    self.qc_thruput[extname] = star_spec.all_hdu[extname].header['ESO QC THRUPUT']
                    self.qc_zpoint[extname] = star_spec.all_hdu[extname].header['ESO QC ZPOINT']
                    self.crpix1[extname] = star_spec.all_hdu[extname].header['CRPIX1']
                    self.crval1[extname] = star_spec.all_hdu[extname].header['CRVAL1']
                    self.cdelt1[extname] = star_spec.all_hdu[extname].header['CDELT1']
                    self.spec_data[extname] = star_spec.all_hdu[extname].data

                    # noise_extname is like IFU.3.NOISE
                    noise_extname = re.sub("DATA", "NOISE", extname)    
                    self.spec_noise[extname] = star_spec.all_hdu[noise_extname].data


            # Set the plotting functions
            self._add_subplots = self._add_subplots
            self._plot = self._data_plot
        else:
            # Set the plotting functions to NODATA ones
            self._add_subplots = self._add_nodata_subplots
            self._plot = self._nodata_plot

    def addSubplots(self, figure):
        self._add_subplots(figure)

    def plotProductsGraphics(self):
        self._plot()

    def plotWidgets(self) :
        widgets = list()
        # Radio button
        self.radiobutton = reflex_plot_widgets.InteractiveRadioButtons(self.axradiobutton, self.setRadioCallback, 
                self.star_data_extnames, 0, title='Standard star selection')
        widgets.append(self.radiobutton)
        return widgets

    def setRadioCallback(self, label) :

        # Setup the image display
        imgdisp = pipeline_display.ImageDisplay()
        imgdisp.setAspect('equal')
        imgdisp.z_lim = (self.image_std_avg[label] - self.image_std_stdev[label], self.image_std_avg[label] + 2 * self.image_std_stdev[label])
        imgdisp.display(self.img_plot, "Median collapsed cube (STD_IMAGE)\nFWHM "+ r"$\simeq$ " +str( numpy.round(self.qc_spat_res[label], 2) )+ "  [arcsec]",                     self._data_plot_get_tooltip(label), self.image_std[label])
        self.img_plot.set_xlabel("pixels")
        self.img_plot.set_ylabel("pixels")

        # Define wave
        pix = numpy.arange(len(self.spec_data[label]))
        wave = self.crval1[label] + pix * self.cdelt1[label]  
          
        # Plot Spectrum
        specdisp = pipeline_display.SpectrumDisplay()
        self.spec_plot.clear()
        specdisp.setLabels(r"$\lambda$["+"$\mu$m]", "Flux (ADU)   [x"+str(self.y_scalefactor)+"]" )
        specdisp.display(self.spec_plot, "Extracted Standard Star Spectrum (STAR_SPEC)", self._data_plot_get_tooltip(label), 
                wave, self.spec_data[label]/self.y_scalefactor) # TO DO: Unify plotting in one function.

        if (self.spec_noise[label] is not None):
            # Overplot the Noise spectrum
            specdisp.overplot(self.spec_plot, wave, self.spec_noise[label]/self.y_scalefactor, 'red')   # TO DO: Unify plotting in one function.
            self.spec_plot.legend(('Flux', 'Noise'))

    def _add_subplots(self, figure):
        gs = gridspec.GridSpec(2, 2)
        self.axradiobutton =    figure.add_subplot(gs[0,0])
        self.img_plot =         figure.add_subplot(gs[0,1])
        self.spec_plot =        figure.add_subplot(gs[1,:])

    def _data_plot_get_tooltip(self, extname):
        # Create the tooltip
        tooltip = " \
            ESO QC NR STD STARS : %f \n \
            ESO QC THRUPUT MEAN : %f \n \
            ESO QC THRUPUT SDV  : %f \n \
            ESO QC STD TRACE    : %f \n \
            ESO QC SPAT RES     : %f \n \
            ESO QC THRUPUT      : %f \n \
            ESO QC ZPOINT       : %f \
            " % (self.qc_nr_std_stars, self.qc_thruput_mean, self.qc_thruput_sdv, self.qc_std_trace[extname], self.qc_spat_res[extname], self.qc_thruput[extname], self.qc_zpoint[extname])
        return tooltip

    def _data_plot(self):
        extname = self.star_data_extnames[0]
        imgdisp = pipeline_display.ImageDisplay()
        imgdisp.setAspect('equal')
        imgdisp.z_lim = (self.image_std_avg[extname] - self.image_std_stdev[extname], self.image_std_avg[extname] + 2 * self.image_std_stdev[extname])
        imgdisp.display(self.img_plot,  "Median collapsed cube (STD_IMAGE)\nFWHM "+ r"$\simeq$ " +str( numpy.round(self.qc_spat_res[extname], 2) )+ "  [arcsec]",                    self._data_plot_get_tooltip(extname), self.image_std[extname])  
        #Please alert if it shuld be latex "\approx" instead of "\simeq".
        self.img_plot.set_xlabel("pixels")
        self.img_plot.set_ylabel("pixels")

        # Define wave
        pix = numpy.arange(len(self.spec_data[extname]))
        wave = self.crval1[extname] + pix * self.cdelt1[extname]  
        
        # Plot Spectrum
        specdisp = pipeline_display.SpectrumDisplay()
        specdisp.setLabels(r"$\lambda$["+"$\mu$m]", "Flux (ADU)   [x"+str(self.y_scalefactor)+"]" )
        specdisp.display(self.spec_plot, "Extracted Standard Star Spectrum (STAR_SPEC)", self._data_plot_get_tooltip(extname), wave, self.spec_data[extname]/self.y_scalefactor) # TO DO: Unify plotting in one function.

        if (self.spec_noise[extname] is not None):
            # Overplot the Noise spectrum
            specdisp.overplot(self.spec_plot, wave, self.spec_noise[extname]/self.y_scalefactor, 'red') # TO DO: Unify plotting in one function.
            self.spec_plot.legend(('Flux', 'Noise'))

    def _add_nodata_subplots(self, figure):
        self.img_plot = figure.add_subplot(1,1,1)

    def _nodata_plot(self):
        # could be moved to reflex library?
        self.img_plot.set_axis_off()
        text_nodata = "Data not found"
        self.img_plot.text(0.1, 0.6, text_nodata, color='#11557c', fontsize=18, ha='left', va='center', alpha=1.0)
        self.img_plot.tooltip = text_nodata

#This is the 'main' function
if __name__ == '__main__':
    from reflex_interactive_app import PipelineInteractiveApp

    # Create interactive application
    interactive_app = PipelineInteractiveApp(enable_init_sop=True)

    # get inputs from the command line
    interactive_app.parse_args()

    #Check if import failed or not
    if not import_success:
        interactive_app.setEnableGUI(False)

    #Open the interactive window if enabled
    if interactive_app.isGUIEnabled():
        #Get the specific functions for this window
        dataPlotManager = DataPlotterManager()

        interactive_app.setPlotManager(dataPlotManager)
        interactive_app.showGUI()
    else:
        interactive_app.set_continue_mode()

    #Print outputs. This is parsed by the Reflex python actor to
    #get the results. Do not remove
    interactive_app.print_outputs()
    sys.exit()



from __future__ import with_statement
import sys

try:
    import numpy
    import os
    import re
    import reflex
    from pipeline_product import PipelineProduct
    import pipeline_display
    import reflex_plot_widgets
    import matplotlib.gridspec as gridspec
    from matplotlib.text import Text
    from pylab import *

    import_success = True
except ImportError:
    import_success = False
    print "Error importing modules pyfits, wx, matplotlib, numpy"

def paragraph(text, width=None):
    """ wrap text string into paragraph
       text:  text to format, removes leading space and newlines
       width: if not None, wraps text, not recommended for tooltips as
              they are wrapped by wxWidgets by default
    """
    import textwrap
    if width is None:
        return textwrap.dedent(text).replace('\n', ' ').strip()
    else:
        return textwrap.fill(textwrap.dedent(text), width=width)

class DataPlotterManager(object):
    # static members
    recipe_name = "kmos_sci_red"
    reconstructed_cat = "SCI_RECONSTRUCTED"
    oh_spec_cat = "OH_SPEC"
    
    IFU_stat_color = {'Active':'summer', 'Locked':'spring', 'NotInPAF':'winter', 'NotInPAF & Locked':'autumn', 'Coll.':'cool', 'Empty':'copper'}

    def setWindowTitle(self):
        return self.recipe_name+"_interactive"

    def setInteractiveParameters(self):
        return [
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="imethod",
                    group="Recons.", description="Interpolation Method (NN, lwNN, swNN, MS, CS)"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="oscan",
                    group="Recons.", description="Apply Overscan Correction"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="xcal_interpolation",
                    group="Recons.", description="Interpolate xcal between rotator angles"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="b_samples",
                    group="Recons.", description="The number of samples in wavelength"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="b_start",
                    group="Recons.", description="The lowest wavelength"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="b_end",
                    group="Recons.", description="The highest wavelength"),

            reflex.RecipeParameter(recipe=self.recipe_name, displayName="fmethod",
                    group="Extr.", description="Fitting Method (gauss, moffat)"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="neighborhoodRange",
                    group="Extr.", description="Range for Neighbors"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="flux",
                    group="Extr.", description="Apply flux conservation"),

            reflex.RecipeParameter(recipe=self.recipe_name, displayName="obj_sky_table",
                    group="Sky Sub.", description="The path to the file with the modified obj/sky associations."),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="background",
                    group="Sky Sub.", description="Apply background removal"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="no_subtract",
                    group="Sky Sub.", description="Don't sky subtract object and references"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="sky_tweak",
                    group="Sky Sub.", description="Use modified sky cube for sky subtraction (TRUE or FALSE)"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="skip_sky_oh_align",
                    group="Sky Sub.", description="Do not align the sky on OH lines - Only is stretch is on (0-no, 1-yes)"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="stretch",
                    group="Sky Sub.", description="Stretch the sky for sky subtraction (0-no, 1-yes)"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="stretch_degree",
                    group="Sky Sub.", description="Stretching polynomial degree."),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="stretch_resampling",
                    group="Sky Sub.", description="Stretching interpolation method (linear / spline)"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="velocity_offset",
                    group="Sky Sub.", description="Specify velocity offset correction in km/s for lambda scale."),
        ]

    def readFitsData(self, fitsFiles):
        # Initialise
        self.files = dict()
        self.oh_spec = dict()
        self.oh_spec["Found"] = False

        # Loop on all FITS files 
        for f in fitsFiles:
            # Use OH_SPEC if found
            if f.category == self.oh_spec_cat :
                oh_spec_file = PipelineProduct(f)
                self.oh_spec["Found"] = True
                self.oh_spec["CRVAL1"] = oh_spec_file.all_hdu[1].header["CRVAL1"]
                self.oh_spec["CDELT1"] = oh_spec_file.all_hdu[1].header["CDELT1"]
                self.oh_spec["Spectrum"] = oh_spec_file.all_hdu[1].data

            # For each reconstructed image
            if f.category == self.reconstructed_cat :
                recons_file = PipelineProduct(f)
                filename = os.path.basename(f.name)
                # Create a Dictionary per file
                self.files[filename] = dict()
                # Loop on extensions
                for recons_ext in recons_file.all_hdu:
                    # EXTNAME is missing in the primary header - Skip it anyway
                    try:
                        extname = recons_ext.header['EXTNAME']
                    except KeyError:
                        continue
                   
                    # Create Entry for the extension
                    self.files[filename][extname]=dict()
                        
                    # Get the IFU number from extname to get the IFU status
                    m = re.search(r"\d+", extname)
                    ifu_number = m.group()
                    self.files[filename][extname]["IFU_NUMBER"] = int(ifu_number)

                    naxis = recons_ext.header['NAXIS']
                    # Set the IFU STATUS
                    arm_key = 'ESO OCS ARM'+ifu_number+' NOTUSED'
                    if (arm_key in recons_file.all_hdu[0].header)  :
                        self.files[filename][extname]["IFU_STATUS"] = recons_file.all_hdu[0].header[arm_key]
                    else :
                        if (naxis == 3):
                            self.files[filename][extname]["IFU_STATUS"] = 'Active'
                        else :
                            self.files[filename][extname]["IFU_STATUS"] = 'Empty'

                    if (naxis == 3):
                        # Get Keyword infos  
                        self.files[filename][extname]["CRPIX3"] = recons_ext.header['CRPIX3']
                        self.files[filename][extname]["CRVAL3"] = recons_ext.header['CRVAL3']
                        self.files[filename][extname]["CDELT3"] = recons_ext.header['CDELT3']
                        self.files[filename][extname]["UNIT"] = recons_ext.header['ESO QC CUBE_UNIT']
                        self.files[filename][extname]["NAME"] = recons_ext.header['ESO OCS ARM' + ifu_number +' NAME']
                        
                        # Fill Spectrum
                        self.files[filename][extname]["Spectrum"] = []
                        for cube_plane in recons_ext.data:
                            cube_plane_nan_free = cube_plane[~numpy.isnan(cube_plane)]
                            if (len(cube_plane_nan_free) > 0):
                                mean = cube_plane_nan_free.mean()
                            else:
                                mean = numpy.nan
                            self.files[filename][extname]["Spectrum"].append(mean)

                        # Fill Collapsed Image
                        collapsed_frame_ext = self._get_collapsed_ext(fitsFiles, f, extname)
                        if collapsed_frame_ext:
                            self.files[filename][extname]["Collapsed"] = collapsed_frame_ext.data

        # If proper files are there...
        if (len(self.files.keys()) > 0):
            # Set the plotting functions
            self._add_subplots = self._add_subplots
            self._plot = self._data_plot
        else:
            self._add_subplots = self._add_nodata_subplots
            self._plot = self._nodata_plot

    # Inputs : all files, reconstructed file, extension
    # Returns the corresponding extension in the collapsed corresponding file 
    # The corresponding file must have the same file name as ref_file, prefixed with make_image_
    # The corresponding extention must have the same EXTNAME keyword
    def _get_collapsed_ext(self, fitsFiles, ref_file, extname):
        ref_file_name = os.path.basename(ref_file.name)
        # Loop on all FITS files 
        for f in fitsFiles:
            filename = os.path.basename(f.name)
            if (filename == "make_image_"+ref_file_name):
                collapsed_frame = PipelineProduct(f)
                for collapsed_frame_ext in collapsed_frame.all_hdu:
                    # EXTNAME is missing in the primary header - Skip it anyway
                    try:
                        coll_extname = collapsed_frame_ext.header['EXTNAME']
                    except KeyError:
                        continue
                    if coll_extname == extname:
                        return collapsed_frame_ext

    def addSubplots(self, figure):
        self._add_subplots(figure)

    def plotProductsGraphics(self):
        self._plot()

    def plotWidgets(self) :
        widgets = list()

        # Files Selector radiobutton
        self.radiobutton = reflex_plot_widgets.InteractiveRadioButtons(self.files_selector, self.setFSCallback, sorted(self.files.keys()), 0, 
                title='Files Selection (Left Mouse Button)')
        widgets.append(self.radiobutton)
        
        self.clickable_ifus =reflex_plot_widgets.InteractiveClickableSubplot(self.ifus_selector, self.setIFUSCallback)
        widgets.append(self.clickable_ifus)

        return widgets

    def extension_has_spectrum(self, filename, extname):
        if ("Spectrum" in self.files[self.selected_file][extname].keys()):
            return True
        else:
            return False

    def setIFUSCallback(self, point) :
        if (1 < point.ydata < 3) :
            extname = "IFU."+str(int((point.xdata/2)+0.5))+".DATA"
            if (self.extension_has_spectrum(self.selected_file, extname)):
                # Update selected extension
                self.selected_extension = extname
                # Redraw IFUs selection
                self._plot_ifus_selector(self.selected_file)
                # Redraw spectrum
                self._plot_spectrum()
                # Redisplay image
                self._disp_image()

    def setFSCallback(self, filename) :
        # Keep track of the selected file
        self.selected_file = filename

        # Check that the new file currently selected extension is valid
        if (not self.extension_has_spectrum(self.selected_file, self.selected_extension)):
            self.selected_extension = self._get_first_valid_extname(self.selected_file)
            self._plot_ifus_selector(self.selected_file)
        # Redraw spectrum
        self._plot_spectrum()
        # Redisplay image
        self._disp_image()
        
    def _add_subplots(self, figure):
        gs = gridspec.GridSpec(3, 2)
        self.files_selector = figure.add_subplot(gs[0,:])
        self.ifus_selector = figure.add_subplot(gs[1,:])
        self.spec_plot = figure.add_subplot(gs[2,0])
        self.collapsed_img = figure.add_subplot(gs[2,1])

    def _data_plot_get_tooltip(self):
        return self.selected_file+" ["+self.selected_extension+"];" +" Object NAME = "+str(  self.files[self.selected_file][self.selected_extension]["NAME"]  )

    def _disp_image(self):
        extension_dict = self.files[self.selected_file][self.selected_extension] 
        self.collapsed_img.clear()
        imgdisp = pipeline_display.ImageDisplay()
        imgdisp.setAspect('equal')
        
        # Only if data
        if 'Collapsed' in extension_dict.keys():
            imgdisp.display(self.collapsed_img, "Collapsed image", self._data_plot_get_tooltip(), extension_dict["Collapsed"])
            self.collapsed_img.set_xlabel("pixels")   # item 6f
            self.collapsed_img.set_ylabel("pixels")   # item 6f
            

    def _plot_spectrum(self):
        extension_dict = self.files[self.selected_file][self.selected_extension] 
        
        # Plot Spectrum (to be contd.)
        self.spec_plot.clear()
        specdisp = pipeline_display.SpectrumDisplay()
        specdisp.setLabels(r"$\lambda$["+"$\mu$m]" + " (blue: Observed; red: OH[arb. units])", 
                            self._process_label(extension_dict["UNIT"]) )
        
        if (self.oh_spec["Found"]):  # Placed before spectrum plotting, so it stays in the background
			# Overplot the OH spectrum
            pix = numpy.arange(len(self.oh_spec["Spectrum"]))
            wave = self.oh_spec["CRVAL1"] + pix * self.oh_spec["CDELT1"]
            oh_flux = self.oh_spec["Spectrum"] / numpy.nanmax(self.oh_spec["Spectrum"]) * 200*numpy.nanmax(extension_dict["Spectrum"]) 
            specdisp.overplot(self.spec_plot, wave, oh_flux, '#ED5D5D')
            #self.spec_plot.legend(('Observed', 'OH'))
            self.spec_plot.fill_between(wave, 0, oh_flux, color='#ED5D5D')
        
        # Define wave
        pix = numpy.arange(len(extension_dict["Spectrum"]))
        wave = extension_dict["CRVAL3"] + pix * extension_dict["CDELT3"]

		# Plot Spectrum (contd.)
        specdisp.display(self.spec_plot, "Spectrum", self._data_plot_get_tooltip(), wave, extension_dict["Spectrum"])



    def _process_label(self, in_label):
        # If known, 'pretty print' the label
        if (in_label == "ERG/sec/cm2/A"):
            return "erg sec" + r"$^{-1}$"+"cm" + r"$^{-2}$" + r"$\AA^{-1}$"
        else:
            return in_label

    def _get_ifu_status_from_file(self, filename, extname):
        cur_status = self.files[filename][extname]["IFU_STATUS"]
        if cur_status == "NotInPAF/Locked" :
            return 'NotInPAF & Locked'
        return cur_status

    def _plot_ifus_selector(self, filename):
        self.ifus_selector.clear()

        # Loop on the different kind of Status to Print the Legend
        self.ifus_selector.imshow(numpy.zeros((1,1)), extent=(1, 9, 8, 10), cmap='summer')
        self.ifus_selector.text(2, 8.5, 'Active', fontsize=11,color='white')
        self.ifus_selector.imshow(numpy.zeros((1,1)), extent=(11, 19, 8, 10), cmap='spring')
        self.ifus_selector.text(12, 8.5, 'Locked', fontsize=11,color='white')
        self.ifus_selector.imshow(numpy.zeros((1,1)), extent=(21, 29, 8, 10), cmap='winter')
        self.ifus_selector.text(22, 8.5, 'NotInPAF', fontsize=11,color='white')
        self.ifus_selector.imshow(numpy.zeros((1,1)), extent=(31, 39, 8, 10), cmap='autumn')
        self.ifus_selector.text(32, 8.5, 'NotInPAF & Locked', fontsize=11,color='white')
        self.ifus_selector.imshow(numpy.zeros((1,1)), extent=(41, 49, 8, 10), cmap='cool')
        self.ifus_selector.text(42, 8.5, 'Collision', fontsize=11,color='white')
        self.ifus_selector.imshow(numpy.zeros((1,1)), extent=(1, 9, 4, 6), cmap='copper')
        self.ifus_selector.text(2, 4.5, 'Empty', fontsize=11,color='white')

        # Display the IFUs selection squares
        box_y_start = 1
        box_y_stop  = 3
        box_xwidth  = 1.5
        for extname in self.files[filename].keys():
            # Compute the IFU number
            ifu_number  = self.files[filename][extname]['IFU_NUMBER']
            # Draw the little IFU image
            box_xstart  = 2 * ifu_number - 1
            box_xstop = box_xstart + box_xwidth

            ifu_status = self._get_ifu_status_from_file(filename, extname)
            self.ifus_selector.imshow(numpy.zeros((1,1)), extent=(box_xstart,box_xstop,box_y_start,box_y_stop), 
                    cmap=self.IFU_stat_color[ifu_status])
            # Write the IFU number in the image
            self.ifus_selector.text(2 * (ifu_number-1) + 1.2, 1.5, str(ifu_number), fontsize=13,color='white')
            # Mark the selected IFU
            if (extname == self.selected_extension):
                self.ifus_selector.imshow(numpy.zeros((1,1)), extent=(box_xstart,box_xstop,0.5,0.7), 
                        cmap=self.IFU_stat_color[ifu_status])

        self.ifus_selector.axis([0,50,0,11])
        self.ifus_selector.set_title("IFU Selection (Mouse middle button)")
        self.ifus_selector.get_xaxis().set_ticks([])
        self.ifus_selector.get_yaxis().set_ticks([])

    # Get the first valid extension name (ie containing a spectrum) in a file - "" if None 
    def _get_first_valid_extname(self, filename):
        for extname in sorted(self.files[filename].keys()):
            if (self.extension_has_spectrum(filename, extname)) :
                return extname
        return ""

    def _data_plot(self):
        # Initial file is the first one
        self.selected_file = self.files.keys()[0]
        self.selected_extension = self._get_first_valid_extname(self.selected_file)
        
        # Plot the IFUS selection
        self._plot_ifus_selector(self.selected_file)

        # Draw Spectrum
        self._plot_spectrum()

        # Display image
        self._disp_image()

    def _add_nodata_subplots(self, figure):
        self.img_plot = figure.add_subplot(1,1,1)

    def _nodata_plot(self):
        # could be moved to reflex library?
        self.img_plot.set_axis_off()
        text_nodata = "Data not found. Input files should contain this" \
                       " type:\n%s" % self.reconstructed_cat
        self.img_plot.text(0.1, 0.6, text_nodata, color='#11557c',
                      fontsize=18, ha='left', va='center', alpha=1.0)
        self.img_plot.tooltip = 'No data found'

#This is the 'main' function
if __name__ == '__main__':
    from reflex_interactive_app import PipelineInteractiveApp

    # Create interactive application
    interactive_app = PipelineInteractiveApp(enable_init_sop=True)

    # get inputs from the command line
    interactive_app.parse_args()

    #Check if import failed or not
    if not import_success:
        interactive_app.setEnableGUI(False)

    #Open the interactive window if enabled
    if interactive_app.isGUIEnabled():
        #Get the specific functions for this window
        dataPlotManager = DataPlotterManager()

        interactive_app.setPlotManager(dataPlotManager)
        interactive_app.showGUI()
    else:
        interactive_app.set_continue_mode()

    #Print outputs. This is parsed by the Reflex python actor to
    #get the results. Do not remove
    interactive_app.print_outputs()
    sys.exit()



#!/usr/bin/python

import sys
import os
import time
from optparse import OptionParser
import pyfits
from pyfits import getheader
import glob
import logging

def multiple_log(message):
    print message
    logging.info(message)

def log_filelist ( filelist ):
    "Log filelist informations"

    if len(filelist) < 1:
        logging.info("Empty Files list")
    else:
        for my_file in filelist:
            logging.info("Name: {0} / n ext: {1} / size: {2} / PRO.CATG: {3} / band: {4}".format(
                    my_file['name'], my_file['n_ext'], my_file['size'], my_file['pro_catg'], my_file['band']))
            logging.info("\tExtnames : "+','.join(my_file['extnames']))
            logging.info("\tAngles : "+','.join(map(str, my_file['angles'])))
    return

#######################################################################################
###     Main Programm
#######################################################################################
# Setup logging
logging.basicConfig(filename='kmos_verify.log', level=logging.DEBUG, datefmt='%m/%d/%Y %I:%M:%S %p', 
        format='%(asctime)s - %(levelname)s - %(message)s')

# Define the command line Options and Parameters
usage = "usage: %prog [options] result_dir"
parser = OptionParser(usage=usage, description="KMOS Calibration Data Verification Script")
parser.add_option("-d", "--description", action="store_true", dest="description", default=False, help="Detailed Description")
(options, args) = parser.parse_args()

# If the description is wished, print it and return
description = '''
\033[1mKMOS Calibration Data Verification Script\033[0m

\033[1mPurpose:\033[0m
The script runs basic verifications on the calibration files generated by mos_calib.py
The results/ directory created by kmos_calib.py si supposed to contain the following files:
- badpixel_dark.fits
- master_dark.fits
- badpixel_flat_[HHH|KKK|HKHKHK|IZIZIZ|YJYJYJ].fits
- det_img_wave_[HHH|KKK|HKHKHK|IZIZIZ|YJYJYJ].fits
- flat_edge_[HHH|KKK|HKHKHK|IZIZIZ|YJYJYJ].fits
- master_flat_[HHH|KKK|HKHKHK|IZIZIZ|YJYJYJ].fits
- lcal_[HHH|KKK|HKHKHK|IZIZIZ|YJYJYJ].fits
- xcal_[HHH|KKK|HKHKHK|IZIZIZ|YJYJYJ].fits
- ycal_[HHH|KKK|HKHKHK|IZIZIZ|YJYJYJ].fits

The scripts checks the consistency between the xcal, ycal and lcal files.

\033[1mVerifications:\033[0m
- Check that there are 15 files
- Check that all bands [H,K,HK,IZ,YJ] have exactly 1 xcal, 1ycal, 1lcal
- Check that all found files have the same size(+/-5%), nb of extensions, angles list, 
    extnames list
- Check that the angles list contains 6 different values (3 times each)
''' 
if options.description:
    print description
    sys.exit(0)

logging.info("Input Directory: {0}".format(args[0]))
logging.info("Description required: {0}".format(options.description))

# Loop on all files in the input directory
files_list = []
for fname in glob.glob(args[0]+"/*cal*.fits"):
    # Open the file
    hdulist = pyfits.open(fname)
    # Initialize the angles list
    angles_list = []
    extnames_list = []
    for ix in range(len(hdulist)):
        if (ix == 0) :
            continue
        angles_list.append(hdulist[ix].header['HIERARCH ESO PRO ROT NAANGLE'])
        extnames_list.append(hdulist[ix].header['EXTNAME'])
    # Store the file informations
    files_list.append({  
            'name': fname, 
            'n_ext': len(hdulist),
            'size': os.path.getsize(fname),
            'pro_catg': hdulist[0].header['HIERARCH ESO PRO CATG'],
            'band': hdulist[0].header['HIERARCH ESO INS GRAT1 ID'],
            'extnames' : extnames_list,
            'angles' : angles_list})
    # Close the file
    hdulist.close()

# Display the stored information
log_filelist(files_list)

# Define expected values
detected_problems = 0 
expected_bands = ['H','K','HK','IZ','YJ']
expected_procatg = ['XCAL', 'YCAL', 'LCAL']
nb_expected_files = 3*len(expected_bands)
nb_expected_angles = 6 ;

# Check that there are 15 files
if len(files_list) == nb_expected_files:
    logging.info("Number of files ... ok")
else:
    multiple_log("Number of files ... Nok - {0} files instead of {1}".format(len(files_list), nb_expected_files))
    detected_problems+=1

# Check that all bands [H,K,HK,IZ,YJ] have exactly 1 xcal, 1ycal, 1lcal
for band in expected_bands:
    for procatg in expected_procatg:
        # Extract current procatg/band files 
        cur_list = [x for x in files_list if x['pro_catg'] == procatg and x['band'] == band]

        # Chaeck that 1 file is found
        if len(cur_list) == 1:
            logging.info("Check if 1 file in band {0} with PRO.CATG {1} ... ok".format(band, procatg))
        else:
            multiple_log("Check if 1 file in band {0} with PRO.CATG {1} ... Nok - {2} files found".format(
                        band, procatg, len(cur_list)))
            detected_problems+=1

# Check that all found files have the same size(+/-5%), nb of extensions, angles list, extnames list
ref_size = files_list[0]['size']
ref_n_ext = files_list[0]['n_ext']
ref_angles = files_list[0]['angles']
ref_extnames = files_list[0]['extnames']
tol = 0.05
for ix in range(len(files_list)):
    if (ix == 0) :
        continue
    # Check size
    if files_list[ix]['size'] > ref_size*0.95 and files_list[ix]['size'] < ref_size*1.05 :
        logging.info("Check if {0} size is the same (+/-5%) as the first file ({1})... ok".format(
                    files_list[ix]['name'], ref_size))
    else:
        multiple_log("Check if {0} size is the same (+/-5%) as the first file ({1})... Nok - {2}".format(
                    files_list[ix]['name'], ref_size, files_list[ix]['size']))
        detected_problems+=1

    # Check nb of extensions
    if files_list[ix]['n_ext'] == ref_n_ext :
        logging.info("Check if {0} nb of extensions is the same as for the first file ({1})... ok".format(
                    files_list[ix]['name'], ref_n_ext))
    else:
        multiple_log("Check if {0} nb of extensions is the same as for the first file ({1})... Nok - {2}".format(
                    files_list[ix]['name'], ref_n_ext, files_list[ix]['n_ext']))
        detected_problems+=1

    # Check Angles list
    if cmp(files_list[ix]['angles'], ref_angles) == 0 :
        logging.info("Check if {0} angles are the same as for the first file ... ok".format(files_list[ix]['name']))
    else:
        multiple_log("Check if {0} angles are the same as for the first file ... Nok - Check the log file".format(
                    files_list[ix]['name']))
        logging.info("\tRef Angles : "+','.join(map(str, ref_angles)))
        logging.info("\tCur Angles : "+','.join(map(str, files_list[ix]['angles'])))
        detected_problems+=1

    # Check extnames list
    if cmp(files_list[ix]['extnames'], ref_extnames) == 0 :
        logging.info("Check if {0} ext names are the same as for the first file ... ok".format(files_list[ix]['name']))
    else:
        multiple_log("Check if {0} ext names are the same as for the first file ... Nok - Check the log file".format(
                    files_list[ix]['name']))
        logging.info("\tRef ExtNames : "+','.join(map(str, ref_extnames)))
        logging.info("\tCur ExtNames : "+','.join(map(str, files_list[ix]['extnames'])))
        detected_problems+=1

# Check that the angles list contains 6 different values (3 times each)
angles = list(set(ref_angles))
if len(angles) == nb_expected_angles:
    logging.info("Number of angles ... ok")
else:
    multiple_log("Number of angles ... Nok - {0} angles instead of {1}".format(len(angles), nb_expected_angles))
    detected_problems+=1

# Check that there are 3 extension per angle
for angle in angles:
    cur_angles = [x for x in ref_angles if x == angle]
    if len(cur_angles) == 3:
        logging.info("Number of extensions for angle {0} ... ok".format(angle))
    else:
        multiple_log("Number of extensions for angle {0} ...  Nok - {1} angles instead of 3".format(angle, len(cur_angles)))
        detected_problems+=1

# Summary
if detected_problems == 0:
    multiple_log("... No problem detected")
else:
    multiple_log("WARNING - {0} problem(s) detected".format(detected_problems))
    



#!/usr/bin/python

import sys
import os
import time
from optparse import OptionParser
import pyfits
from pyfits import getval
from pyfits import getheader
import glob
import logging
import subprocess
from subprocess import Popen, list2cmdline
import shutil


#######################################################################################
###     Files Selection function
#######################################################################################
def keyword_based_file_selection( filelist, keyword ):
    "Choose Which Template needs to be reduced"

    # Get the different keyword values encountered
    keyword_values = list(set(map(lambda x: x[keyword], filelist)))

    # One set - nothing to choose
    if len(keyword_values) == 1:
        return keyword_values[0]

    logging.info("User selection amongst {0} choices".format(len(keyword_values)))
    for index in range(len(keyword_values)):
        # Extract this file list
        extracted_files = [x for x in filelist if x[keyword] == keyword_values[index]]
        if keyword == 'tpl_start':
            print_filelist_tpl_candidate(index, extracted_files)
        elif keyword == 'obs_start': 
            print_filelist_ob_candidate(index, extracted_files)
        else:
            raise NameError("Unsupported keyword {0}".format(keyword))

    index = input('Choose Data Set Index: ')
    if index >= 0 and index < len(keyword_values):
        return keyword_values[index]
    else :
        raise NameError("Entered index out of range")

#######################################################################################
###     Printing / Logging Functions
#######################################################################################
def print_filelist_tpl_candidate( index, filelist ):
    "Prints relevant informations about a Template for user choice"

    if len(filelist) < 1:
        message = "\tTemplate Index: {0} - Empty Files list".format(index)
    else:
        firstfile = filelist[0]
        do_catgs = list(set(map(lambda x: do_catg_from_file(x), filelist)))
        message = "\tTemplate Index: {0} - {1} {2}/{3} ".format(index,firstfile['tpl_start'], len(filelist),
                firstfile['tpl_nexp'])+','.join(do_catgs) 
    multiple_log(message)
    return

def print_filelist_ob_candidate( index, filelist ):
    "Prints relevant informations about an OB for user choice"

    if len(filelist) < 1:
        message = "\tObs Block Index: {0} - Empty Files list".format(index)
    else:
        # Extract dark files - Count different TPL STARTs
        filelist_darks = [x for x in filelist if x['tpl_id'] == "KMOS_spec_cal_dark"]
        n_dark_tpls = len(list(set(map(lambda x: x['tpl_start'], filelist_darks))))

        # Extract Flat files - Count different TPL STARTs - Get different bands
        filelist_flats = [x for x in filelist if x['tpl_id'] == "KMOS_spec_cal_calunitflat"]
        n_flat_tpls = len(list(set(map(lambda x: x['tpl_start'], filelist_flats))))
        flat_bands = list(set(map(lambda x: x['band'], filelist_flats)))
        # Extract Wave files - Count different TPL STARTs - Get different bands
        filelist_waves = [x for x in filelist if x['tpl_id'] == "KMOS_spec_cal_wave"]
        n_wave_tpls = len(list(set(map(lambda x: x['tpl_start'], filelist_waves))))
        wave_bands = list(set(map(lambda x: x['band'], filelist_waves)))
        
        message = "\tObs Block Index: {0} ({1}) - {2} Dark TPLs, {3} Flat TPLs ({4}), {5} Wavecal TPLs ({6})".format(index,
            filelist[0]['obs_start'], n_dark_tpls, n_flat_tpls, ','.join(flat_bands), n_wave_tpls, ','.join(wave_bands)) 
    multiple_log(message)
    return

def multiple_log_ob_selected( filelist ):
    "Prints relevant informations about a selected OB"

    if len(filelist) < 1:
        message = "\t-> Selected : Empty Files list"
    else:
        firstfile = filelist[0]
        do_catgs = list(set(map(lambda x: do_catg_from_file(x), filelist)))
        message = "\t-> Selected : {0}".format(firstfile['obs_start'])
    multiple_log(message)
    return

def multiple_log_tpl_selected( filelist ):
    "Prints relevant informations about a selected Template"

    if len(filelist) < 1:
        message = "\t-> Selected : Empty Files list"
    else:
        firstfile = filelist[0]
        do_catgs = list(set(map(lambda x: do_catg_from_file(x), filelist)))
        message = "\t-> Selected : {0} {1}/{2} ".format(firstfile['tpl_start'], len(filelist),
                firstfile['tpl_nexp'])+','.join(do_catgs) 
    multiple_log(message)
    return

#######################################################################################
###     Various Utilities
#######################################################################################
def tpl_id_to_recipe_name( tpl_id ) :
    "Get the recipe name from the tpl.id"
    if tpl_id == "KMOS_spec_cal_dark" :
        return "kmos_dark"
    if tpl_id == "KMOS_spec_cal_calunitflat" :
        return "kmos_flat"
    if tpl_id == "KMOS_spec_cal_wave" :
        return "kmos_wave_cal"
    return "Unknown_TPL_ID"
    
def do_catg_from_file(file_entry):
    "Get the DO_CATG for a given file"

    if file_entry['tpl_id'] == "KMOS_spec_cal_dark" :
        return "DARK"
    if file_entry['tpl_id'] == "KMOS_spec_cal_calunitflat" :
        if file_entry['dpr_type'] == "FLAT,OFF" : 
            return "FLAT_OFF"
        if file_entry['dpr_type'] == "FLAT,LAMP" : 
            return "FLAT_ON"
    if file_entry['tpl_id'] == "KMOS_spec_cal_wave" :
        if file_entry['dpr_type'] == "WAVE,OFF" : 
            return "ARC_OFF"
        if file_entry['dpr_type'] == "WAVE,LAMP" : 
            return "ARC_ON"
    raise NameError("Unknown DO_CATG")
    return

#######################################################################################
###     SOF Creation
#######################################################################################
def create_sof( filelist, mydir ):
    "Creates a SOF file in mydir directory"
    
    ###############
    # Verifications
    if len(filelist) < 1 :
        raise NameError("Empty List")
        return
    # Same band ?
    if len(set(map(lambda x: x['band'], filelist))) != 1 :
        raise NameError("Different bands in this sof")
        return
    # Same tpl_id ?
    if len(set(map(lambda x: x['tpl_id'], filelist))) != 1 :
        raise NameError("Different TPL_IDs in this sof")
        return
    # Same tpl_start ?
    if len(set(map(lambda x: x['tpl_start'], filelist))) != 1 :
        raise NameError("Different TPL_STARTs in this sof")
        return
    ###############

    # SOF file name
    sof_basename = tpl_id_to_recipe_name(filelist[0]['tpl_id'])+"_"+filelist[0]['tpl_start']+".sof"
    sof_name = mydir+"/"+sof_basename

    # Open the file
    sof = open(sof_name, "wb")

    # Write RAW Files
    for file_entry in filelist:
        sof.write(file_entry['name']+" "+do_catg_from_file(file_entry)+"\n")

    band_lc = filelist[0]['band'].lower()
    band_3uc = filelist[0]['band']*3

    # Add Calibrations
    if filelist[0]['tpl_id'] == "KMOS_spec_cal_calunitflat" :
        sof.write(os.getcwd()+"/badpixel_dark.fits BADPIXEL_DARK\n")

    if filelist[0]['tpl_id'] == "KMOS_spec_cal_wave" :
        sof.write(os.environ['KMOS_CALIB']+"/kmos_wave_ref_table.fits  REF_LINES\n")
        sof.write(os.environ['KMOS_CALIB']+"/kmos_wave_band.fits       WAVE_BAND\n")
        sof.write(os.environ['KMOS_CALIB']+"/kmos_ar_ne_list_"+band_lc+".fits    ARC_LIST\n")
        sof.write(os.getcwd()+"/"+filelist[0]['band']+"/flat_edge_"+band_3uc+".fits   FLAT_EDGE\n")
        sof.write(os.getcwd()+"/"+filelist[0]['band']+"/xcal_"+band_3uc+".fits        XCAL\n")
        sof.write(os.getcwd()+"/"+filelist[0]['band']+"/ycal_"+band_3uc+".fits        YCAL\n")

    # Close sof file
    sof.close()
    return sof_basename
   
#######################################################################################
###     Commands Execution functions
#######################################################################################
def exec_commands_seq(cmds):
    "Exec commands sequentially"
    for cmd in cmds:
        multiple_log("\tRun in {0} : {1} ...".format(cmd['dir'], list2cmdline(cmd['cmd'])))
        cwd = os.getcwd()
        os.chdir(cmd['dir'])
        subprocess.check_call(cmd['cmd'], stdout=open(os.devnull, 'wb'), stderr=open(os.devnull, 'wb'))
        os.chdir(cwd)

def exec_commands_par(cmds):
    "Exec commands in parallel in multiple process"
    def done(p):
        return p.poll() is not None
    def success(p):
        return p.returncode == 0
    def fail():
        sys.exit(1)

    max_task = os.sysconf('SC_NPROCESSORS_ONLN')
    processes = []
    while True:
        while cmds and len(processes) < max_task:
            cmd = cmds.pop()
            multiple_log("\tRun in {0} : {1} ...".format(cmd['dir'], list2cmdline(cmd['cmd'])))
            processes.append(Popen(cmd['cmd'],cwd=cmd['dir'],stdout=open(os.devnull, 'wb'),stderr=open(os.devnull, 'wb')))

        for p in processes:
            if done(p):
                if success(p):
                    processes.remove(p)
                else:
                    fail()

        if not processes and not cmds:
            break
        else:
            time.sleep(0.05)

def exec_commands(cmds, par):
    "Exec commands"
    # Empty list
    if not cmds: 
        return 
    if par:
        exec_commands_par(cmds)
    else:
        exec_commands_seq(cmds)

def multiple_log(message):
    print message
    logging.info(message)

#######################################################################################
###     Main Programm
#######################################################################################
# Setup logging
logging.basicConfig(filename='kmos_calib.log', level=logging.DEBUG, datefmt='%m/%d/%Y %I:%M:%S %p', 
        format='%(asctime)s - %(levelname)s - %(message)s')

# Define the command line Options and Parameters

# Whenever argparse is available on the pipeline ws, use argparse instead of deprecated optparse
# import argparse
# parser = argparse.ArgumentParser(description='KMOS Calibration')
# parser.add_argument('dir', metavar='dir', type=str, help='The reference directory')
# parser.add_argument('--parallel', action='store_true', default=False, help='Parallel execution of esorex')
# args = parser.parse_args()
usage = "usage: %prog [options] data_dir"
parser = OptionParser(usage=usage, description="KMOS Calibration Data Generation Script")
parser.add_option("-p", "--parallel", action="store_true", dest="parallel", default=False, help="Parallel execution of esorex")
parser.add_option("-d", "--description", action="store_true", dest="description", default=False, help="Detailed Description")
parser.add_option("-b", "--band", default="All", help="Band that needs to be reduced (H, K, HK, YJ, IZ) [default: %default]")
(options, args) = parser.parse_args()

# If the description is wished, print it and return
description = '''
\033[1mKMOS Calibration Data Generation Script\033[0m

\033[1mPurpose:\033[0m
Execute kmos_dark/_flat/_wave_cal on the data from the 3 relevant templates in the 5 bands in order to create the calibration files set needed for the image reconstruction on the instrument workstation.
The wished 37 files are:
- badpixel_dark.fits
- master_dark.fits
- badpixel_flat_[HHH|KKK|HKHKHK|IZIZIZ|YJYJYJ].fits
- det_img_wave_[HHH|KKK|HKHKHK|IZIZIZ|YJYJYJ].fits
- flat_edge_[HHH|KKK|HKHKHK|IZIZIZ|YJYJYJ].fits
- master_flat_[HHH|KKK|HKHKHK|IZIZIZ|YJYJYJ].fits
- lcal_[HHH|KKK|HKHKHK|IZIZIZ|YJYJYJ].fits
- xcal_[HHH|KKK|HKHKHK|IZIZIZ|YJYJYJ].fits
- ycal_[HHH|KKK|HKHKHK|IZIZIZ|YJYJYJ].fits

\033[1mUsage:\033[0m
- Create a new directory and cd to it (mkdir tmp ; cd tmp)
- Run the command, using the full path of the raw files directory:
  kmos_calib.py -p /.../raw_files/ (full path)
- Among the KMOS*.fits files, if there are several ObsBlocks, the user will be given the choice to select one
- Among the selected ObsBlock, if there are several templates executed, the user will be given the choice to select one
- Wait till the execution is finished (~9' in parallel, ~35' in sequential on wu1pl)
- The calibration files are in ./results/*.fits

\033[1mPrerequisites:\033[0m
- The script must be run in an empty directory
- KMOS_CALIB = /cal/kmos/cal
- ESOREX_PLUGIN_DIR = /usr/lib64/esopipes-plugins
- esorex needs to be in the PATH
- The Parallel option only works on Linux (use of specific command to get the number of processors)
- The argument is a reference directory containing KMOS FITS file (full path).

\033[1mAlgorithm\033[0m
1. Create the set of files filelist, composed by all files:
  - that are in the specified directory
  - that match KMOS*.fits 
  - are from one of the 3 relevant templates (use TPL.ID)
2. Extract from filelist (into filelist_ob) the files that are from the 
  ObsBlock chosen by the user (use OBS.START)
3. Extract from filelist_ob the files from the dark template.
   If several template executions (TPL.START), ask the user to choose one.
4. For each different band (INS.GRAT1.ID) identified in ObsSet:
  a. Extract from filelist_ob the files of this band and from the flat template
     If several template executions (TPL.START), ask the user to choose one.
  b. Extract from filelist_ob the files of this band and from the wave_cal template.
     If several template executions (TPL.START), ask the user to choose one.
5. Produce the sof files and the execution scripts
6. Run the scripts in Parallel or not
7. Verifications on the products
''' 
if options.description:
    print description
    sys.exit(0)

logging.info("Input Directory: {0}".format(args[0]))
logging.info("Parallel Mode: {0}".format(options.parallel))
logging.info("Description required: {0}".format(options.description))
logging.info("Wished Band: {0}".format(options.band))

# Environment Variables
my_var_name = "KMOS_CALIB"
my_var_value = "/cal/kmos/cal"
if not my_var_name in os.environ :
    multiple_log(my_var_name+" is not set - set it to "+my_var_value)
    os.environ[my_var_name] = my_var_value
else:
    multiple_log("Use "+my_var_name+"="+os.environ[my_var_name])

my_var_name = "ESOREX_PLUGIN_DIR"
my_var_value = "/usr/lib64/esopipes-plugins"
if not my_var_name in os.environ :
    multiple_log(my_var_name+" is not set - set it to "+my_var_value)
    os.environ[my_var_name] = my_var_value
else:
    multiple_log("Use "+my_var_name+"="+os.environ[my_var_name])

# Loop on all files in the input directory
filelist_dir = []
for fname in glob.glob(args[0]+"/KMOS*.fits"):
    # Read the Primary header
    hdu = getheader(fname, 0)
    tpl_id = hdu['HIERARCH ESO TPL ID']
    # Only keep the proper TPL.ID
    if tpl_id in ["KMOS_spec_cal_dark","KMOS_spec_cal_calunitflat","KMOS_spec_cal_wave"] :
        filelist_dir.append({   'name': fname, 
                                'tpl_id': tpl_id, 
                                'tpl_start': hdu['HIERARCH ESO TPL START'],
                                'tpl_nexp': hdu['HIERARCH ESO TPL NEXP'],
                                'tpl_expno': hdu['HIERARCH ESO TPL EXPNO'],
                                'dpr_type': hdu['HIERARCH ESO DPR TYPE'],
                                'obs_start': hdu['HIERARCH ESO OBS START'],
                                'band': hdu['HIERARCH ESO INS GRAT1 ID']})

# Check that files have been selected
if len(filelist_dir) < 1:
    raise NameError("Could not find any relevant file")

# Select the Wished ObsBlock
obs_start = keyword_based_file_selection(filelist_dir, 'obs_start')

# Extract the Chosen ObsBlock files
filelist_ob = [x for x in filelist_dir if x['obs_start'] == obs_start]
multiple_log_ob_selected(filelist_ob) 

### Dark Template Selection ###
multiple_log("Dark files Selection")

# Extract the dark files   
dark_files = [x for x in filelist_ob if x['tpl_id'] == "KMOS_spec_cal_dark"]
# Check that some darks are found
if len(dark_files) == 0 :
    raise NameError("No Dark Files Found - Choose another OB")
# Which TPL START is whished ?
tpl_start = keyword_based_file_selection(dark_files, 'tpl_start')
# Extract the Chosen Template files 
dark_files_tpl = [x for x in dark_files if x['tpl_start'] == tpl_start]
# Sort
dark_files_tpl = sorted(dark_files_tpl, key=lambda x: x['name'])
multiple_log_tpl_selected(dark_files_tpl) 

# Get List of bands
bands = list(set(map(lambda x: x['band'], filelist_ob)))
multiple_log("Identified Bands: "+','.join(bands))

# Initialize selected_sets
selected_sets = []

# Loop on the different bands encountered 
for band_curr in bands:
    # Skip the bands that are not wished
    if options.band != 'All' and options.band != band_curr:
        continue
    
    ### Flatfield Template Selection ###
    multiple_log("Flat files Selection in Band: {0}".format(band_curr))

    # Extract the flat files in this band  
    flat_files_band = [x for x in filelist_ob if x['band'] == band_curr and x['tpl_id'] == "KMOS_spec_cal_calunitflat"]
    # Check that some flats are found
    if len(flat_files_band) == 0 :
        multiple_log("No Flat Field in this band - Skip it")
        continue
    # Which TPL START is whished ?
    tpl_start = keyword_based_file_selection(flat_files_band, 'tpl_start')
    # Extract the Chosen Template files 
    flat_files_tpl = [x for x in flat_files_band if x['tpl_start'] == tpl_start]
    flat_files_tpl = sorted(flat_files_tpl, key=lambda x: x['name'])
    # Log selected file lists
    multiple_log_tpl_selected(flat_files_tpl) 

    ### Wavecal Template Selection ###
    multiple_log("Wavecal files Selection in Band: {0}".format(band_curr))
    # Extract the wavecal files in this band  
    wavecal_files_band = [x for x in filelist_ob if x['band'] == band_curr and x['tpl_id'] == "KMOS_spec_cal_wave"]
    # Check that some wavecal are found
    if len(wavecal_files_band) == 0 :
        continue
    # Which TPL START is whished ?
    tpl_start = keyword_based_file_selection(wavecal_files_band, 'tpl_start')
    # Extract the Chosen Template files 
    wavecal_files_tpl = [x for x in wavecal_files_band if x['tpl_start'] == tpl_start]
    wavecal_files_tpl = sorted(wavecal_files_tpl, key=lambda x: x['name'])
    # Log selected file lists
    multiple_log_tpl_selected(wavecal_files_tpl) 

    # Store the flat_files_tpl and wavecal_files_tpl
    selected_sets.append({  'band': band_curr, 
                            'flat_files': flat_files_tpl, 
                            'wavecal_files': wavecal_files_tpl})
   
# Create Sub-Directories
for set_curr in selected_sets:
    # Skip the bands that are not wished
    if options.band != 'All' and options.band != set_curr['band']:
        continue
    
    # Create the subdirectory
    my_dir = set_curr['band']
    os.mkdir(my_dir) 

# Reduce Dark
my_dir = '.'
sof_name = create_sof(dark_files_tpl, my_dir)
recipe_name = tpl_id_to_recipe_name(dark_files_tpl[0]['tpl_id'])
log_file = "esorex_"+recipe_name+".log"
dark_cmd = ['esorex', '--suppress-prefix=TRUE', '--log-file='+log_file, '--log-dir=.', recipe_name, sof_name]
cmds = [ {'cmd': dark_cmd, 'dir': my_dir} ]
print("Dark Data Reduction")
exec_commands(cmds, options.parallel)

# Reduce Flats
cmds = []
for set_curr in selected_sets:
    # Skip the bands that are not wished
    if options.band != 'All' and options.band != set_curr['band']:
        continue

    # Generate exection command
    my_dir = set_curr['band']
    sof_name = create_sof(set_curr['flat_files'], my_dir)
    recipe_name = tpl_id_to_recipe_name(set_curr['flat_files'][0]['tpl_id'])
    log_file = "esorex_"+recipe_name+".log"
    flat_cmd = ['esorex', '--suppress-prefix=TRUE', '--log-file='+log_file, '--log-dir=.', recipe_name, sof_name]
    cmds.append({'cmd': flat_cmd, 'dir': my_dir})
print("Flatfield Data Reduction")
exec_commands(cmds, options.parallel)

# Reduce Wavecal
cmds = []
for set_curr in selected_sets:
    # Skip the bands that are not wished
    if options.band != 'All' and options.band != set_curr['band']:
        continue

    # Generate exection command
    my_dir = set_curr['band']
    sof_name = create_sof(set_curr['wavecal_files'], my_dir)
    recipe_name = tpl_id_to_recipe_name(set_curr['wavecal_files'][0]['tpl_id'])
    log_file = "esorex_"+recipe_name+".log"
    wavecal_cmd = ['esorex', '--suppress-prefix=TRUE', '--log-file='+log_file, '--log-dir=.', recipe_name, sof_name]
    cmds.append({'cmd': wavecal_cmd, 'dir': my_dir})
print("Wavelength Calibration Data Reduction")
exec_commands(cmds, options.parallel)

# Create results directory
if os.path.exists('results'):
    answer = input('Directory results already exists - Proceed (1) or Abort (0) ?')
    if answer != 1:
        sys.exit(0)
else:
    os.mkdir('results') 

# Collect results
os.rename(os.getcwd()+"/master_dark.fits", os.getcwd()+"/results/master_dark.fits")
os.rename(os.getcwd()+"/badpixel_dark.fits", os.getcwd()+"/results/badpixel_dark.fits")
for set_curr in selected_sets:
    # Skip the bands that are not wished
    if options.band != 'All' and options.band != set_curr['band']:
        continue

    # Collect the band results
    my_dir = set_curr['band']
    band_3uc = set_curr['band']*3
    filenames = [   "xcal_{0}.fits".format(band_3uc), 
                    "ycal_{0}.fits".format(band_3uc), 
                    "lcal_{0}.fits".format(band_3uc),
                    "master_flat_{0}.fits".format(band_3uc),
                    "badpixel_flat_{0}.fits".format(band_3uc),
                    "det_img_wave_{0}.fits".format(band_3uc),
                    "flat_edge_{0}.fits".format(band_3uc)]
    for filename in filenames :
        src_file = os.getcwd()+"/"+my_dir+"/"+filename
        dest_file = os.getcwd()+"/results/"+filename
        if os.path.isfile(src_file) :
            os.rename(src_file, dest_file)



from __future__ import with_statement
import sys

try:
    import numpy
    import reflex
    from pipeline_product import PipelineProduct
    import pipeline_display
    import reflex_plot_widgets
    import matplotlib.gridspec as gridspec
    import_success = True
except ImportError:
    import_success = False
    print "Error importing modules pyfits, wx, matplotlib, numpy"

def paragraph(text, width=None):
    """ wrap text string into paragraph
       text:  text to format, removes leading space and newlines
       width: if not None, wraps text, not recommended for tooltips as
              they are wrapped by wxWidgets by default
    """
    import textwrap
    if width is None:
        return textwrap.dedent(text).replace('\n', ' ').strip()
    else:
        return textwrap.fill(textwrap.dedent(text), width=width)


class DataPlotterManager(object):
    # static members
    recipe_name = "kmos_wave_cal"
    det_img_wave_cat = "DET_IMG_WAVE"
    lcal_cat = "LCAL"

    def setWindowTitle(self):
        return self.recipe_name+"_interactive"

    def setInteractiveParameters(self):
        return [
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="order",
                    group="Wavelength Calibration", description="The wavelength polynomial order [0,7]"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="dev_flip",
                    group="Wavelength Calibration", description="TRUE if the wavelengths are ascending on the detector from top to bottom"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="dev_disp",
                    group="Wavelength Calibration", description="The expected dispersion of the wavelength in microns/pixel"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="suppress_extension",
                    group="Wavelength Calibration", description="Suppress arbitrary filename extension"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="b_samples",
                    group="Wavelength Calibration", description="The number of samples in wavelength for the reconstructed cube"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="b_start",
                    group="Wavelength Calibration", description="The lowest wavelength to take into account when reconstructing"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="b_end",
                    group="Wavelength Calibration", description="The highest wavelength to take into account when reconstructing"),
        ]

    def readFitsData(self, fitsFiles):
        self.frames = dict()
        for f in fitsFiles:
            self.frames[f.category] = PipelineProduct(f)

        # Two cases: the file category is found or not found.
        # Define the plotting functions in both cases
        if self.det_img_wave_cat in self.frames:
            # Get the wished files
            det_img_wave = self.frames[self.det_img_wave_cat]
            self.det_img_wave_name = det_img_wave.fits_file.name

            # Get the angle values
            self.angles_list = dict()
            for i in range(18):
                angle = det_img_wave.readKeyword('ESO PRO ROT NAANGLE', i+1)

                if not (angle in self.angles_list) :
                    self.angles_list[angle] = []
                self.angles_list[angle].append(i+1)

            # Sorted angles list
            self.sorted_angles = self.angles_list.keys()
            self.sorted_angles.sort() 

            # Read the Plotting Data
            self.argon_pos_data = []
            self.argon_fwhm_data = []
            self.neon_pos_data = []
            self.neon_fwhm_data = []
            for i in range(18):
                key1 = det_img_wave.readKeyword('ESO QC ARC AR POS MEAN', i+1)
                key2 = det_img_wave.readKeyword('ESO QC ARC AR FWHM MEAN', i+1)
                ar_vscale = det_img_wave.readKeyword('ESO QC ARC AR VSCALE', i+1)

                key3 = det_img_wave.readKeyword('ESO QC ARC NE POS MEAN', i+1)
                key4 = det_img_wave.readKeyword('ESO QC ARC NE FWHM MEAN', i+1)
                ne_vscale = det_img_wave.readKeyword('ESO QC ARC NE VSCALE', i+1)
           
                # Correct Key2 and Key4 units
                key2 /= ar_vscale
                key4 /= ne_vscale

                # Fill the Argon Data for the scatter plot
                self.argon_pos_data.append(key1)
                self.argon_fwhm_data.append(key2)
                
                # Fill the Neon Data for the scatter plot
                self.neon_pos_data.append(key3)
                self.neon_fwhm_data.append(key4)

            # Read the images
            self.images = []
            for i in range(18):
                det_img_wave.readImage(i+1)
                self.images.append(det_img_wave.image) 

            # Set the plotting functions
            self._add_subplots = self._add_subplots
            self._plot = self._data_plot
        else:
            # Set the plotting functions to NODATA ones
            self._add_subplots = self._add_nodata_subplots
            self._plot = self._nodata_plot

    def addSubplots(self, figure):
        self._add_subplots(figure)

    def plotProductsGraphics(self):
        self._plot()

    def plotWidgets(self) :
        widgets = list()

        # Radio button
        self.radiobutton = reflex_plot_widgets.InteractiveRadioButtons(self.axradiobutton, self.setRadioCallback, self.sorted_angles, 0, title='Angle selection')
        
        widgets.append(self.radiobutton)
        return widgets

    def setRadioCallback(self, label) :
        # Get Extensions as list
        extensions = self.angles_list[float(label)] 
        if (len(extensions) == 3):
            # Setup the image display
            for i in range(3):
                imgdisp = pipeline_display.ImageDisplay()
                imgdisp.setAspect('equal')
                imgdisp.display(self.img_plot[i], "Extension {0}".format(extensions[i]), self._data_plot_get_tooltip(extensions[i]), self.images[extensions[i]-1])
        else:
            print "Wrong number of extensions for this angle"

    def _add_subplots(self, figure):
      
        gs = gridspec.GridSpec(5, 3)
        self.img_plot = []
        for i in range(3):
            self.img_plot.append(figure.add_subplot(gs[0:2,i]))
        self.axradiobutton = figure.add_subplot(gs[3:,0])
        self.argon_plot = figure.add_subplot(gs[3,1:])
        self.neon_plot = figure.add_subplot(gs[4,1:])

    def _data_plot_get_tooltip(self, extension):
        # Create the tooltip
        tooltip = " \
                ESO QC ARC AR POS MEAN : %f \n \
                ESO QC ARC AR FWHM MEAN : %f \n \
                ESO QC ARC NE POS MEAN : %f \n \
                ESO QC ARC NE FWHM MEAN : %f \
        " % (self.argon_pos_data[extension-1],self.argon_fwhm_data[extension-1],self.neon_pos_data[extension-1],self.neon_fwhm_data[extension-1])
        return tooltip

    def _data_plot(self):
        # Get Extensions as list
        extensions = self.angles_list[self.sorted_angles[0]] 
        if (len(extensions) == 3):
            # Setup the image display
            for i in range(3):
                imgdisp = pipeline_display.ImageDisplay()
                imgdisp.setAspect('equal')
                imgdisp.display(self.img_plot[i], "Extension {0}".format(extensions[i]), self._data_plot_get_tooltip(extensions[i]), self.images[extensions[i]-1])
        else:
            print "Wrong number of extensions for this angle"

        # Define x
        x = numpy.linspace(1, 18, num=18)
        
        # Plot Argon and Neon plots
        scadsp = pipeline_display.ScatterDisplay()
        scadsp.display(self.argon_plot, "Argon Lines", "Mean Argon line positional offset, with error bars equal to the FWH", x, self.argon_pos_data,
                self.argon_fwhm_data)

        scadsp = pipeline_display.ScatterDisplay()
        scadsp.setLabels('File Extensions 1 -> 18','Line Pos. (pix)')
        scadsp.display(self.neon_plot, "Neon Lines", "Mean Neon line positional offset, with error bars equal to the FWH", x, self.neon_pos_data,
                self.neon_fwhm_data)

    def _add_nodata_subplots(self, figure):
        self.img_plot = figure.add_subplot(1,1,1)

    def _nodata_plot(self):
        # could be moved to reflex library?
        self.img_plot.set_axis_off()
        text_nodata = "Data not found. Input files should contain this" \
                       " type:\n%s" % self.det_img_wave_cat
        self.img_plot.text(0.1, 0.6, text_nodata, color='#11557c',
                      fontsize=18, ha='left', va='center', alpha=1.0)
        self.img_plot.tooltip = 'No data found'

#This is the 'main' function
if __name__ == '__main__':
    from reflex_interactive_app import PipelineInteractiveApp

    # Create interactive application
    interactive_app = PipelineInteractiveApp(enable_init_sop=True)

    # get inputs from the command line
    interactive_app.parse_args()

    #Check if import failed or not
    if not import_success:
        interactive_app.setEnableGUI(False)

    #Open the interactive window if enabled
    if interactive_app.isGUIEnabled():
        #Get the specific functions for this window
        dataPlotManager = DataPlotterManager()

        interactive_app.setPlotManager(dataPlotManager)
        interactive_app.showGUI()
    else:
        interactive_app.set_continue_mode()

    #Print outputs. This is parsed by the Reflex python actor to
    #get the results. Do not remove
    interactive_app.print_outputs()
    sys.exit()
